import 'dart:async';
import 'package:pip_services3_commons/pip_services3_commons.dart';
import 'package:pip_services3_rpc/pip_services3_rpc.dart';
import 'package:customers_microservice_dart/src/data/version1/data.dart';
import 'package:customers_microservice_dart/src/logic/logic.dart';

import 'ICustomersClientV1.dart';

class CustomersDirectClientV1 extends DirectClient<ICustomersController> implements ICustomersClientV1
{
	CustomersDirectClientV1()
		: super()
	{
		dependencyResolver.put('controller', Descriptor('eic-customers', 'controller', '*', '*', '1.0'));
	}

	@override
	Future<CustomerV1> createCustomer(String correlationId, CustomerV1 customer) async 
	{
		var timing = instrument(correlationId, 'customers.create_customer');
		var result = await controller.createCustomer(correlationId, customer);
		timing.endTiming();
		return result;
	}

	@override
	Future<CustomerV1> updateCustomer(String correlationId, CustomerV1 customer) async 
	{
		var timing = instrument(correlationId, 'customers.update_customer');
		var result = await controller.updateCustomer(correlationId, customer);
		timing.endTiming();
		return result;
	}

	@override
	Future<CustomerV1> deleteCustomerById(String correlationId, String id) async 
	{
		var timing = instrument(correlationId, 'customers.delete_customer_by_id');
		var result = await controller.deleteCustomerById(correlationId, id);
		timing.endTiming();
		return result;
	}

	@override
	Future<CustomerV1> getCustomerById(String correlationId, String id) async 
	{
		var timing = instrument(correlationId, 'customers.get_customer_by_id');
		var result = await controller.getCustomerById(correlationId, id);
		timing.endTiming();
		return result;
	}

	@override
	Future<DataPage<CustomerV1>> getCustomers(String correlationId, FilterParams filter, PagingParams paging, SortParams sort) async 
	{
		var timing = instrument(correlationId, 'customers.get_customers');
		var result = await controller.getCustomers(correlationId, filter, paging, sort);
		timing.endTiming();
		return result;
	}
}
