import 'dart:async';
import 'dart:convert';
import 'package:pip_services3_commons/pip_services3_commons.dart';
import 'package:pip_services3_rpc/pip_services3_rpc.dart';
import 'package:customers_microservice_dart/src/data/version1/data.dart';
import 'ICustomersClientV1.dart';

class CustomersHttpClientV1 extends CommandableHttpClient implements ICustomersClientV1
{
	CustomersHttpClientV1([config])
		: super('v1/customers')
	{
		if (config != null)
		{
			configure(ConfigParams.fromValue(config));
		};
	}

	@override
	Future<CustomerV1> createCustomer(String correlationId, CustomerV1 customer) async {
	    var result = await callCommand('create_customer', correlationId, { 'customer': customer });
	    if (result == null) return null;
	    var item = CustomerV1();
	    item.fromJson(json.decode(result));
	    return item;
	}

	@override
	Future<CustomerV1> updateCustomer(String correlationId, CustomerV1 customer) async {
	    var result = await callCommand('update_customer', correlationId, { 'customer': customer });
	    if (result == null) return null;
	    var item = CustomerV1();
	    item.fromJson(json.decode(result));
	    return item;
	}

	@override
	Future<CustomerV1> deleteCustomerById(String correlationId, String id) async {
	    var result = await callCommand(
	        'delete_customer_by_id', correlationId, { 'customer_id': id });
	    if (result == null) return null;
	    var item = CustomerV1();
	    item.fromJson(json.decode(result));
	    return item;
	}

	@override
	Future<CustomerV1> getCustomerById(String correlationId, String id) async {
	    var result = await callCommand(
	        'get_customer_by_id', correlationId, { 'customer_id': id});
	    if (result == null) return null;
	    var item = CustomerV1();
	    item.fromJson(json.decode(result));
	    return item;
	}

	@override
	Future<DataPage<CustomerV1>> getCustomers(String correlationId, FilterParams filter, PagingParams paging, SortParams sort) async {
	    var result = await callCommand(
	      'get_customers',
	      correlationId,
	      { 
	        'filter': filter, 
	        'paging': paging,
	        'sort': sort
	      }
	    );
	    return DataPage<CustomerV1>.fromJson(json.decode(result), (item) {
	        var customer = CustomerV1();
	        customer.fromJson(item);
	        return customer;
	    });
	}
}
