import 'dart:async';
import 'package:pip_services3_commons/pip_services3_commons.dart';
import '../../src/data/version1/data.dart';
import 'logic.dart';

class CustomersCommandSet extends CommandSet
{
	ICustomersController _controller;

	CustomersCommandSet(ICustomersController controller)
	{
		_controller = controller;

		addCommand(_makeCreateCustomerCommand());
		addCommand(_makeUpdateCustomerCommand());
		addCommand(_makeDeleteCustomerByIdCommand());
		addCommand(_makeGetCustomerByIdCommand());
		addCommand(_makeGetCustomersCommand());
	}

	ICommand _makeCreateCustomerCommand()
	{
	    return Command(
	        'create_customer',
	        ObjectSchema(true).withRequiredProperty('customer', CustomerV1Schema()),
	        (String correlationId, Parameters args)
	        {
	            var customer = CustomerV1();
	            customer.fromJson(args.get('customer'));
	            return _controller.createCustomer(correlationId, customer);
	        });
	}

	ICommand _makeUpdateCustomerCommand()
	{
	    return Command(
	       'update_customer',
	       ObjectSchema(true)
	            .withRequiredProperty('customer', CustomerV1Schema()),
	       (String correlationId, Parameters args)
	       {
	           var customer = CustomerV1();
	           customer.fromJson(args.get('customer'));
	           return _controller.updateCustomer(correlationId, customer);
	       });
	}

	ICommand _makeDeleteCustomerByIdCommand()
	{
	    return Command(
	       'delete_customer_by_id',
	       ObjectSchema(true).withRequiredProperty('customer_id', TypeCode.String),
	       (String correlationId, Parameters args)
	       {
	           var customerId = args.getAsString('customer_id');
	           return _controller.deleteCustomerById(correlationId, customerId);
	       });
	}

	ICommand _makeGetCustomerByIdCommand()
	{
	    return Command(
	        'get_customer_by_id',
	        ObjectSchema(true).withRequiredProperty('customer_id', TypeCode.String),
	        (String correlationId, Parameters args)
	        {
	            var customerId = args.getAsString('customer_id');
	            return _controller.getCustomerById(correlationId, customerId);
	        });
	}

	ICommand _makeGetCustomersCommand()
	{
	    return Command(
	        'get_customers',
	        ObjectSchema(true)
	            .withOptionalProperty('filter', FilterParamsSchema())
	            .withOptionalProperty('paging', PagingParamsSchema()),
	        (String correlationId, Parameters args)
	        {
	            var filter = FilterParams.fromValue(args.get('filter'));
	            var paging = PagingParams.fromValue(args.get('paging'));
	            return _controller.getCustomers(correlationId, filter, paging, null);
	        });
	}
}
