import 'dart:async';
import 'package:pip_services3_commons/pip_services3_commons.dart';
import '../data/version1/data.dart';
import '../persistence/ICustomersPersistence.dart';
import 'CustomersCommandSet.dart';
import 'ICustomersController.dart';

class CustomersController implements ICustomersController, IConfigurable, IReferenceable, ICommandable
{
	ICustomersPersistence _persistence;
	CustomersCommandSet _commandSet;

	CustomersController()
	{
	}

	@override
	void configure(ConfigParams config)
	{
	}

	@override
	void setReferences(IReferences references)
	{
	    _persistence = references.getOneRequired<ICustomersPersistence >(
	        Descriptor('eic-customers', 'persistence', '*', '*', '1.0'));
	}

	@override
	CommandSet getCommandSet()
	{
	    _commandSet ??= CustomersCommandSet(this);
	    return _commandSet;
	}

	@override
	Future<CustomerV1> createCustomer(String correlationId, CustomerV1 customer) {
	    customer.id = customer.id ?? IdGenerator.nextLong();
	
	    return _persistence.create(correlationId, customer);
	}

	@override
	Future<CustomerV1> updateCustomer(String correlationId, CustomerV1 customer) {
	    return _persistence.update(correlationId, customer);
	}

	@override
	Future<CustomerV1> deleteCustomerById(String correlationId, String id) {
	    return _persistence.deleteById(correlationId, id);
	}

	@override
	Future<CustomerV1> getCustomerById(String correlationId, String id) {
	    return _persistence.getOneById(correlationId, id);
	}

	@override
	Future<DataPage<CustomerV1>> getCustomers(String correlationId, FilterParams filter, PagingParams paging, SortParams sort) {
	    return _persistence.getPageByFilter(correlationId, filter, paging, sort);
	}
}
