import 'package:test/test.dart';
import 'package:pip_services3_commons/pip_services3_commons.dart';
import 'package:customers_microservice_dart/customers_microservice.dart';
import './CustomersClientV1Fixture.dart';

void main() {
  group('CustomersDirectClientV1', () {
    CustomersMemoryPersistence persistence;
    CustomersController controller;
    CustomersDirectClientV1 client;
    CustomersClientV1Fixture fixture;

    setUp(() async {
      persistence = CustomersMemoryPersistence();
      persistence.configure(ConfigParams());

      controller = CustomersController();
      controller.configure(ConfigParams());

      client = CustomersDirectClientV1();

      var references = References.fromTuples([
        Descriptor('eic-customers', 'persistence', 'memory', 'default', '1.0'),
        persistence,
        Descriptor('eic-customers', 'controller', 'default', 'default', '1.0'),
        controller,
        Descriptor('eic-customers', 'client', 'direct', 'default', '1.0'),
        client
      ]);

      controller.setReferences(references);
      client.setReferences(references);

      fixture = CustomersClientV1Fixture(client);

      await persistence.open(null);
    });

    tearDown(() async {
      await persistence.close(null);
    });

    test('Create Customer', () async {
      await fixture.It_Should_Create_Customer();
    });

    test('Update Customer', () async {
      await fixture.It_Should_Update_Customer();
    });

    test('Delete Customer', () async {
      await fixture.It_Should_Delete_Customer();
    });

    test('Get Customer By Id', () async {
      await fixture.It_Should_Get_Customer_By_Id();
    });
  });
}
