import 'dart:async';

import 'package:test/test.dart';
import 'package:pip_services3_commons/pip_services3_commons.dart';
import 'package:customers_microservice_dart/customers_microservice.dart';

import './CustomersClientV1Fixture.dart';

var httpConfig = ConfigParams.fromTuples([
  'connection.protocol',
  'http',
  'connection.host',
  'localhost',
  'connection.port',
  3001
]);

void main() {
  group('CustomersHttpClientV1', () {
    CustomersMemoryPersistence persistence;
    CustomersController controller;
    CustomersHttpServiceV1 service;
    CustomersHttpClientV1 client;
    CustomersClientV1Fixture fixture;

    setUp(() async {
      persistence = CustomersMemoryPersistence();
      persistence.configure(ConfigParams());

      controller = CustomersController();
      controller.configure(ConfigParams());

      service = CustomersHttpServiceV1();
      service.configure(httpConfig);

      client = CustomersHttpClientV1();
      client.configure(httpConfig);
      var references = References.fromTuples([
        Descriptor('eic-customers', 'persistence', 'memory', 'default', '1.0'),
        persistence,
        Descriptor('eic-customers', 'controller', 'default', 'default', '1.0'),
        controller,
        Descriptor('eic-customers', 'service', 'http', 'default', '1.0'),
        service,
        Descriptor('eic-customers', 'client', 'http', 'default', '1.0'),
        client
      ]);

      controller.setReferences(references);
      service.setReferences(references);
      client.setReferences(references);
      
      fixture = CustomersClientV1Fixture(client);
      
      await persistence.open(null);
      await service.open(null);
      await client.open(null);
    });

    tearDown(() async {
      await client.close(null);
      await service.close(null);
      await persistence.close(null);
      await Future.delayed(Duration(milliseconds: 2000));
    });

    test('Create Customer', () async {
      await fixture.It_Should_Create_Customer();
    });

    test('Update Customer', () async {
      await fixture.It_Should_Update_Customer();
    });

    test('Delete Customer', () async {
      await fixture.It_Should_Delete_Customer();
    });

    test('Get Customer By Id', () async {
      await fixture.It_Should_Get_Customer_By_Id();
    });
  });
}
