import 'package:test/test.dart';
import 'package:pip_services3_commons/pip_services3_commons.dart';
import 'package:customers_microservice_dart/customers_microservice.dart';
import '../data/TestModel.dart';

void main() {
  group('CustomersController', () {
    CustomersMemoryPersistence _persistence;
    CustomersController _controller;

    setUp(() async {
      _persistence = CustomersMemoryPersistence();
      _persistence.configure(ConfigParams());

      _controller = CustomersController();
      _controller.configure(ConfigParams());

      var references = References.fromTuples([
        Descriptor('eic-customers', 'persistence', 'memory', 'default', '1.0'),
        _persistence,
        Descriptor('eic-customers', 'controller', 'default', 'default', '1.0'),
        _controller
      ]);

      _controller.setReferences(references);

      await _persistence.open(null);
    });

    tearDown(() async {
      await _persistence.close(null);
    });

    test('Create Customer', () async {
      // arrange
      var customer = TestModel.createCustomer();

      // act
      var result = await _controller.createCustomer(null, customer);

      // assert
      TestModel.assertEqual(customer, result);
    });

    test('Update Customer', () async {
      // arrange
      var customer = await _controller.createCustomer(null, TestModel.createCustomer());

      // act
      customer.first_name = RandomText.word();

      var result = await _controller.updateCustomer(null, customer);

      // assert
      TestModel.assertEqual(customer, result);
    });

    test('Delete Customer', () async {
      // arrange
      var customer = await _controller.createCustomer(null, TestModel.createCustomer());

      // act
      var deleted_customer = await _controller.deleteCustomerById(null, customer.id);

      var result = await _controller.getCustomerById(null, customer.id);

      // assert
      TestModel.assertEqual(customer, deleted_customer);
      expect(result, isNull);
    });

    test('Get Customer By Id', () async {
      // arrange
      var customer = await _controller.createCustomer(null, TestModel.createCustomer());

      // act
      var result = await _controller.getCustomerById(null, customer.id);

      // assert
      TestModel.assertEqual(customer, result);
    });
  });
}
