import 'package:test/test.dart';
import 'package:pip_services3_commons/pip_services3_commons.dart';

import 'package:customers_microservice_dart/customers_microservice.dart';

import '../data/TestModel.dart';

final CUSTOMER1 = TestModel.createCustomer();
final CUSTOMER2 = TestModel.createCustomer();
final CUSTOMER3 = TestModel.createCustomer();

class CustomersPersistenceFixture {
  ICustomersPersistence _persistence;

  CustomersPersistenceFixture(ICustomersPersistence persistence) {
    expect(persistence, isNotNull);
    _persistence = persistence;
  }

  void _testCreateCustomers() async {
    // Create the first customer
    var customer = await _persistence.create(null, CUSTOMER1);
    TestModel.assertEqual(CUSTOMER1, customer);

    // Create the second customer
    customer = await _persistence.create(null, CUSTOMER2);
    TestModel.assertEqual(CUSTOMER2, customer);

    // Create the third customer
    customer = await _persistence.create(null, CUSTOMER3);
    TestModel.assertEqual(CUSTOMER3, customer);
  }

  void testCrudOperations() async {
    CustomerV1 customer1;

    // Create items
    await _testCreateCustomers();

    // Get all Customers
    var page = await _persistence.getPageByFilter(null, FilterParams(), PagingParams(), SortParams([]));
    expect(page, isNotNull);
    expect(page.data.length, 3);

    customer1 = page.data[0];

    // Update the customer
    customer1.first_name = 'ABC';

    var customer = await _persistence.update(null, customer1);
    expect(customer, isNotNull);
    expect(customer1.id, customer.id);
    expect('ABC', customer.first_name);

    // Delete the customer
    customer = await _persistence.deleteById(null, customer1.id);
    expect(customer, isNotNull);
    expect(customer1.id, customer.id);

    // Try to get deleted customer
    customer = await _persistence.getOneById(null, customer1.id);
    expect(customer, isNull);
  }

  void testGetWithFilters() async {
    // Create items
    await _testCreateCustomers();

    // Filter by id
    var page = await _persistence.getPageByFilter(null, 
      FilterParams.fromTuples(['id', CUSTOMER1.id]), PagingParams(), null);
    expect(page.data.length, 1);
  }
}
