import 'dart:convert';
import 'package:test/test.dart';
import 'package:http/http.dart' as http;
import 'package:pip_services3_commons/pip_services3_commons.dart';
import 'package:customers_microservice_dart/customers_microservice.dart';

import '../../data/TestModel.dart';

final CUSTOMER1 = TestModel.createCustomer();
final CUSTOMER2 = TestModel.createCustomer();

var httpConfig = ConfigParams.fromTuples([
  'connection.protocol',
  'http',
  'connection.host',
  'localhost',
  'connection.port',
  3000
]);

void main() {
  group('CustomersHttpServiceV1', () {
    CustomersMemoryPersistence persistence;
    CustomersController controller;
    CustomersHttpServiceV1 service;
    http.Client rest;
    String url;

    setUp(() async {
      url = 'http://localhost:3000';
      rest = http.Client();

      persistence = CustomersMemoryPersistence();
      persistence.configure(ConfigParams());

      controller = CustomersController();
      controller.configure(ConfigParams());

      service = CustomersHttpServiceV1();
      service.configure(httpConfig);

      var references = References.fromTuples([
        Descriptor('eic-customers', 'persistence', 'memory', 'default', '1.0'),
        persistence,
        Descriptor('eic-customers', 'controller', 'default', 'default', '1.0'),
        controller,
        Descriptor('eic-customers', 'service', 'http', 'default', '1.0'),
        service
      ]);

      controller.setReferences(references);
      service.setReferences(references);

      await persistence.open(null);
      await service.open(null);
    });

    tearDown(() async {
      await service.close(null);
      await persistence.close(null);
    });

    test('CRUD Operations', () async {
      CustomerV1 customer1;

      // Create the first customer
      var resp = await rest.post(url + '/v1/customers/create_customer',
          headers: {'Content-Type': 'application/json'},
          body: json.encode({'customer': CUSTOMER1}));
      var customer = CustomerV1();
      customer.fromJson(json.decode(resp.body));
      TestModel.assertEqual(CUSTOMER1, customer);

      // Create the second customer
      resp = await rest.post(url + '/v1/customers/create_customer',
          headers: {'Content-Type': 'application/json'},
          body: json.encode({'customer': CUSTOMER2}));
      customer = CustomerV1();
      customer.fromJson(json.decode(resp.body));
      TestModel.assertEqual(CUSTOMER2, customer);

      // Get all customers
      resp = await rest.post(url + '/v1/customers/get_customers',
          headers: {'Content-Type': 'application/json'},
          body: json
              .encode({'filter': FilterParams(), 'paging': PagingParams()}));
      var page = DataPage<CustomerV1>.fromJson(json.decode(resp.body),
          (item) {
        var customer = CustomerV1();
        customer.fromJson(item);
        return customer;
      });
      expect(page, isNotNull);
      expect(page.data.length, 2);

      customer1 = page.data[0];

      // Update the customer
      customer1.first_name = 'ABC';

      resp = await rest.post(url + '/v1/customers/update_customer',
          headers: {'Content-Type': 'application/json'},
          body: json.encode({'customer': customer1}));
      customer = CustomerV1();
      customer.fromJson(json.decode(resp.body));
      expect(customer, isNotNull);
      expect(customer1.id, customer.id);
      expect('ABC', customer.first_name);

      // Delete the customer
      resp = await rest.post(url + '/v1/customers/delete_customer_by_id',
          headers: {'Content-Type': 'application/json'},
          body: json.encode({'customer_id': customer1.id}));
      customer = CustomerV1();
      customer.fromJson(json.decode(resp.body));
      expect(customer, isNotNull);
      expect(customer1.id, customer.id);

      // Try to get deleted customer
      resp = await rest.post(url + '/v1/customers/get_customer_by_id',
          headers: {'Content-Type': 'application/json'},
          body: json.encode({'customer_id': customer1.id}));
      expect(resp.body, isEmpty);
    });
  });
}
